@@ -1,4 +1,4 @@
-from typing import Awaitable, Callable, List, Tuple, TypeVar, Union
+from typing import Awaitable, Callable, Tuple, TypeVar, Union
 
 from aioredis.commands import Redis
 
@@ -9,7 +9,6 @@ TRedis = TypeVar("TRedis", bound=Redis)
 
 BytesOrStr = Union[bytes, str]
 AioredisAddress = Union[str, Tuple[str, int]]
-SlotsResponse = List[List]
 CommandsFactory = Callable[[AbcConnection], TRedis]
 PoolerBatchCallback = Callable[[AbcPool], Awaitable[None]]
 PoolCreator = Callable[[AioredisAddress], Awaitable[AbcPool]]
